//
//  Char.swift
//  TowerDefense
//
//  Created by 杉本裕樹 on 2016/03/13.
//  Copyright © 2016年 MyName. All rights reserved.
//

import UIKit
import SpriteKit
import GameplayKit

class Char: SKSpriteNode {
    override init(texture: SKTexture?, color: UIColor, size: CGSize) {
        super.init(texture: texture, color: color, size: size)
        isUserInteractionEnabled = true
    }
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    var level = 1
    var didTouched: (() -> ())?
    
    lazy var stateMachine: GKStateMachine = {
        let stateMachine = GKStateMachine(states: [
            AttackState(char: self),
            StayState(char: self)
            ])
        stateMachine.enterState(AttackState.self)
        return stateMachine
    }()
    
    class CharState: GKState {
        weak var char: Char?
        var power: Int {
            return char?.power ?? 0
        }
        
        init(char: Char?) {
            self.char = char
        }
        
        func enableToAttack() -> Bool {
            return false
        }
    }
    
    class AttackState: CharState {
        override func enableToAttack() -> Bool {
            return true
        }
    }
    
    class AngryState: CharState {
        var enteredDate = Date()
        override var power: Int {
            return char?.power ?? 0
        }
        
        override func enableToAttack() -> Bool {
            return true
        }
        
        override func didEnter(withPreviousState previousState: GKState?) {
            enteredDate = Date()
        }
        
        override func update(withDeltaTime seconds: TimeInterval) {
            if Date().timeIntervalSince(enteredDate) > 10 {
                char?.stateMachine.enterState(AttackState.self)
            }
        }
    }
    
    class StayState: CharState {
        var enteredDate = Date()
        override func didEnter(withPreviousState previousState: GKState?) {
            enteredDate = Date()
        }
        
        override func update(withDeltaTime seconds: TimeInterval) {
            if Date().timeIntervalSince(enteredDate) > 0.2 {
                char?.stateMachine.enterState(AttackState.self)
            }
        }
        
        override func enableToAttack() -> Bool {
            return false
        }
    }
    var power: Int {
        return 0
    }
    var cost: Int {
        return 10
    }
    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        didTouched?()
    }
}

class Char1: Char {
    init() {
        super.init(texture: SKTexture(imageNamed: "Char1"), color: UIColor.clear, size: CGSize(width: 32, height: 32))
    }
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override var power: Int {
        return 1 * level
    }
    override var cost: Int {
        return 10
    }
}

class Char2: Char {
    init() {
        super.init(texture: SKTexture(imageNamed: "Char2"), color: UIColor.clear, size: CGSize(width: 32, height: 32))
    }
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override var power: Int {
        return 2 * level
    }
    override var cost: Int {
        return 20
    }
}

class Char3: Char {
    init() {
        super.init(texture: SKTexture(imageNamed: "Char3"), color: UIColor.clear, size: CGSize(width: 32, height: 32))
    }
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override var power: Int {
        return 3 * level
    }
    override var cost: Int {
        return 30
    }
}
