//
//  SetCharScene.swift
//  TowerDefense
//
//  Created by 杉本裕樹 on 2016/04/23.
//  Copyright © 2016年 MyName. All rights reserved.
//

import UIKit
import SpriteKit
import ReplayKit

class SetCharScene: SKScene {
    var stage = 1
    var coin = 100
    var charData = [(factory: CharFactory, position: CGPoint)]()
    
    override func didMove(to view: SKView) {
        physicsWorld.gravity = CGVector(dx: 0, dy: 0)
        let fieldImageLength = view.frame.width / 10
        let field = FieldFactory().createField(view.frame.size, fieldImageLength: fieldImageLength)
        field.nodes.forEach {
            addChild($0)
        }
        
        let buttonRect = CGRect(x: self.frame.maxX - 115,
                                y: self.frame.maxY - 49,
                                width: 110, height: 34)
        let button = Button(text: "ゲーム開始", rect: buttonRect, afterTap: {
            let scene = GameScene(fileNamed: "GameScene")
            scene?.stage = self.stage
            scene?.scaleMode = .resizeFill
            view.presentScene(scene)
            scene?.createChars(self.charData)
        })
        addChild(button)
        
        if !RPScreenRecorder.shared().isRecording {
            RPScreenRecorder.shared().startRecording(withMicrophoneEnabled: false, handler: { error in
            })
        }
    }
    
    func createChars(_ charData: [(factory: CharFactory, position: CGPoint)]) {
        charData.forEach {
            let char = $0.factory.createChar()
            char.position = $0.position
            char.physicsBody = SKPhysicsBody(rectangleOf: char.size)
            addChild(char)
            self.charData.append($0)
            let factory = $0.factory
            char.didTouched = {
                self.charDidTouched(char, factory: factory)
            }
        }
    }

    func charDidTouched(_ char: Char, factory: CharFactory) {
        let alert = UIAlertController(title: nil, message: "どのキャラクターを設置しますか?",
                                      preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "アップグレード", style: .default, handler: { _ in
            self.coin -= char.cost
            char.level += 1
        }))
        alert.addAction(UIAlertAction(title: "売却", style: .default, handler: { _ in
            let index = self.charData.index { $0.factory === factory }!
            self.charData.remove(at: index)
            self.coin += char.cost / 2
            char.removeFromParent()
        }))
        alert.addAction(UIAlertAction(title: "キャンセル", style: .cancel, handler: nil))
        let rootViewController = UIApplication.shared.delegate?.window??.rootViewController
        rootViewController?.present(alert, animated: true, completion: nil)
    }
    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        if touches.count == 1, let point = touches.first?.location(in: self) {
            let alert = UIAlertController(title: nil, message: "どのキャラクターを設置しますか?",
                                          preferredStyle: .alert)
            let rootViewController = UIApplication.shared.delegate?.window??.rootViewController
            alert.addAction(UIAlertAction(title: "Char1を設置", style: .default, handler: { _ in
                let factory = Char1Factory()
                let char = factory.createChar()
                char.position = point
                self.coin -= char.cost
                self.addChild(char)
                self.charData.append((factory: factory, position: point))
                char.didTouched = {
                    self.charDidTouched(char, factory: factory)
                }
            }))
            alert.addAction(UIAlertAction(title: "Char2を設置", style: .default, handler: { _ in
                let factory = Char2Factory()
                let char = factory.createChar()
                char.position = point
                self.coin -= char.cost
                self.addChild(char)
                self.charData.append((factory: factory, position: point))
                char.didTouched = {
                    self.charDidTouched(char, factory: factory)
                }
            }))
            alert.addAction(UIAlertAction(title: "Char3を設置", style: .default, handler: { _ in
                let factory = Char3Factory()
                let char = Char3()
                char.position = point
                self.coin -= char.cost
                self.addChild(char)
                self.charData.append((factory: factory, position: point))
                char.didTouched = {
                    self.charDidTouched(char, factory: factory)
                }
            }))
            alert.addAction(UIAlertAction(title: "キャンセル", style: .cancel, handler: nil))
            rootViewController?.present(alert, animated: true, completion: nil)
        }
    }
}
